@@ -1,5 +1,6 @@
 # Copyright (C) 2018 The Qt Company Ltd.
 # SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+from __future__ import annotations
 
 import functools
 import os
@@ -11,8 +12,8 @@ from ..log import log
 from ..config import config
 from ..options import OPTION
 from ..utils import (copydir, copyfile, copy_qt_metatypes,
-                     download_and_extract_7z, filter_match, makefile)
-from .. import PYSIDE, SHIBOKEN, PYSIDE_WINDOWS_BIN_TOOLS
+                     download_and_extract_7z, filter_match, makefile, in_coin)
+from .. import PYSIDE, SHIBOKEN, PYSIDE_WINDOWS_BIN_TOOLS, PYSIDE_MULTIMEDIA_LIBS
 
 
 def prepare_packages_win32(pyside_build, _vars):
@@ -116,7 +117,7 @@ def prepare_packages_win32(pyside_build, _vars):
             target = f"{{st_build_dir}}/{{st_package_name}}/scripts/{script}"
             copyfile(src, target, force=False, _vars=_vars)
 
-        for script_dir in ("qtpy2cpp_lib", "deploy_lib", "project"):
+        for script_dir in ("qtpy2cpp_lib", "deploy_lib", "project_lib"):
             src = f"{{install_dir}}/bin/{script_dir}"
             target = f"{{st_build_dir}}/{{st_package_name}}/scripts/{script_dir}"
             # Exclude subdirectory tests
@@ -152,10 +153,9 @@ def prepare_packages_win32(pyside_build, _vars):
 
         # <install>/share/{st_package_name}/glue/* ->
         #   <setup>/{st_package_name}/glue
-        copydir(
-            "{install_dir}/share/{st_package_name}/glue",
-            destination_dir / "glue",
-            _vars=_vars)
+        for glue in ['glue', 'doc']:
+            source = "{install_dir}/share/{st_package_name}/" + glue
+            copydir(source, destination_dir / glue, _vars=_vars)
 
         # <source>/pyside6/{st_package_name}/support/* ->
         #   <setup>/{st_package_name}/support/*
@@ -167,7 +167,7 @@ def prepare_packages_win32(pyside_build, _vars):
         # <source>/pyside6/{st_package_name}/QtAsyncio/* ->
         #   <setup>/{st_package_name}/QtAsyncio/*
         copydir(
-            f"{{site_packages_dir}}/{{st_package_name}}/QtAsyncio",
+            "{site_packages_dir}/{st_package_name}/QtAsyncio",
             "{st_build_dir}/{st_package_name}/QtAsyncio",
             _vars=_vars)
 
@@ -217,8 +217,7 @@ msvc_redist = [
 
 
 def copy_msvc_redist_files(destination_dir):
-    in_coin = os.environ.get('COIN_LAUNCH_PARAMETERS', None)
-    if in_coin is None:
+    if not in_coin():
         log.info("Qt dependency DLLs (MSVC redist) will not be copied.")
         return
 
@@ -244,18 +243,15 @@ def copy_msvc_redist_files(destination_dir):
         raise FileNotFoundError(msg)
 
 
-def copy_qt_dependency_dlls(_vars, destination_qt_dir, artifacts):
+def download_qt_dependency_dlls(_vars, destination_qt_dir, artifacts):
     # Extract Qt dependency dlls when building on Qt CI.
-    in_coin = os.environ.get('COIN_LAUNCH_PARAMETERS', None)
-    if in_coin is None:
+    if not in_coin():
         log.info("Qt dependency DLLs will not be downloaded and extracted.")
         return
 
     with tempfile.TemporaryDirectory() as temp_path:
         redist_url = "https://download.qt.io/development_releases/prebuilt/vcredist/"
-        zip_file = "pyside_qt_deps_64_2019.7z"
-        if "{target_arch}".format(**_vars) == "32":
-            zip_file = "pyside_qt_deps_32_2019.7z"
+        zip_file = "pyside_qt_deps_681_64_2022.7z"
         try:
             download_and_extract_7z(redist_url + zip_file, temp_path)
         except Exception as e:
@@ -284,35 +280,31 @@ def copy_qt_artifacts(pyside_build, destination_qt_dir, copy_pdbs, _vars):
         copy_translations = False
         copy_qt_conf = False
         copy_qt_permanent_artifacts = False
-        copy_clang = True
 
     # <qt>/bin/*.dll and Qt *.exe -> <setup>/{st_package_name}
-    qt_artifacts_permanent = [
-        "opengl*.dll",
-        "designer.exe",
-        "linguist.exe",
-        "lrelease.exe",
-        "lupdate.exe",
-        "lconvert.exe",
-        "qtdiag.exe"
-    ]
+    qt_multimedia_artifacts_permanent = [f"{lib}-*.dll" for lib in PYSIDE_MULTIMEDIA_LIBS]
 
-    # Choose which EGL library variants to copy.
-    qt_artifacts_egl = [
-        "libEGL{}.dll",
-        "libGLESv2{}.dll"
-    ]
-    if pyside_build.qtinfo.build_type != 'debug_and_release':
-        egl_suffix = '*'
-    elif pyside_build.debug:
-        egl_suffix = 'd'
-    else:
-        egl_suffix = ''
-    qt_artifacts_egl = [a.format(egl_suffix) for a in qt_artifacts_egl]
+    def qt_rhi_artifacts_permanent():
+        result = ["opengl*.dll"]
+        if pyside_build.qtinfo.build_type != 'debug_and_release':
+            suffix = '*'
+        elif pyside_build.debug:
+            suffix = 'd'
+        else:
+            suffix = ''
+        result.append(f"libEGL{suffix}.dll")  # Needed in Qt 6 (ANGLE)?
+        result.append(f"libGLESv2{suffix}.dll")
+        return result
 
     if copy_qt_permanent_artifacts:
-        artifacts = qt_artifacts_permanent + qt_artifacts_egl
-        copy_qt_dependency_dlls(_vars, destination_qt_dir, artifacts)
+        download_qt_dependency_dlls(_vars, destination_qt_dir, qt_rhi_artifacts_permanent())
+        if not copydir("{qt_bin_dir}", destination_qt_dir,
+                       _filter=qt_multimedia_artifacts_permanent,
+                       recursive=False, _vars=_vars):
+            msg = "The multimedia artifacts were not found."
+            log.warning(msg)
+            if in_coin():  # Allow local developer builds without -DQT_DEPLOY_FFMPEG...
+                raise FileNotFoundError(msg)
 
     # <qt>/bin/*.dll and Qt *.pdbs -> <setup>/{st_package_name} part two
     # File filter to copy only debug or only release files.
