@@ -782,10 +782,7 @@ PyTypeObject *getTypeForQObject(const QObject *cppSelf)
     if (existing != nullptr)
         return reinterpret_cast<PyObject *>(existing)->ob_type;
     // Find the best match (will return a PySide type)
-    auto *sbkObjectType = Shiboken::ObjectType::typeForTypeName(typeName(cppSelf));
-    if (sbkObjectType != nullptr)
-        return reinterpret_cast<PyTypeObject *>(sbkObjectType);
-    return nullptr;
+    return Shiboken::ObjectType::typeForTypeName(typeName(cppSelf));
 }
 
 PyObject *getWrapperForQObject(QObject *cppSelf, PyTypeObject *sbk_type)
