@@ -123,19 +123,24 @@ void HeaderGenerator::generateSmartPointerClass(TextStream &s,
     doGenerateClass(s, classContext);
 }
 
+QString HeaderGenerator::headerGuard(const QString &className)
+{
+    return "SBK_"_L1 + getFilteredCppSignatureString(className.toUpper()) + "_H"_L1;
+}
+
 void HeaderGenerator::doGenerateClass(TextStream &s, const GeneratorContext &classContext) const
 {
-    const AbstractMetaClassCPtr metaClass = classContext.metaClass();
+    const AbstractMetaClassCPtr &metaClass = classContext.metaClass();
 
     // write license comment
     s << licenseComment();
 
     QString wrapperName = classContext.effectiveClassName();
-    QString outerHeaderGuard = getFilteredCppSignatureString(wrapperName);
+    QString outerHeaderGuard = headerGuard(classContext.effectiveClassName());
 
     // Header
-    s << "#ifndef SBK_" << outerHeaderGuard << "_H\n";
-    s << "#define SBK_" << outerHeaderGuard << "_H\n\n";
+    s << "#ifndef " << outerHeaderGuard << '\n'
+      << "#define " << outerHeaderGuard << "\n\n";
 
     if (!avoidProtectedHack())
         s << protectedHackDefine;
@@ -148,14 +153,14 @@ void HeaderGenerator::doGenerateClass(TextStream &s, const GeneratorContext &cla
     if (classContext.useWrapper())
         writeWrapperClass(s, wrapperName, classContext);
 
-    s << "#endif // SBK_" << outerHeaderGuard << "_H\n\n";
+    s << "#endif // " << outerHeaderGuard << '\n';
 }
 
 void HeaderGenerator::writeWrapperClass(TextStream &s,
                                         const QString &wrapperName,
                                         const GeneratorContext &classContext) const
 {
-    const auto metaClass = classContext.metaClass();
+    const auto &metaClass = classContext.metaClass();
 
     if (avoidProtectedHack()) {
         const auto includeGroups = classIncludes(metaClass);
@@ -192,23 +197,22 @@ void HeaderGenerator::writeInheritedWrapperClassDeclaration(TextStream &s,
                                                             const GeneratorContext &classContext) const
 {
     const QString wrapperName = classContext.effectiveClassName();
-    const QString innerHeaderGuard =
-        getFilteredCppSignatureString(wrapperName).toUpper();
+    const QString innerHeaderGuard = headerGuard(wrapperName);
 
-    s << "#  ifndef SBK_" << innerHeaderGuard << "_H\n"
-      << "#  define SBK_" << innerHeaderGuard << "_H\n\n"
+    s << "#  ifndef " << innerHeaderGuard << '\n'
+      << "#  define " << innerHeaderGuard << "\n\n"
       << "// Inherited base class:\n";
 
     writeWrapperClassDeclaration(s, wrapperName, classContext);
 
-    s << "#  endif // SBK_" << innerHeaderGuard << "_H\n\n";
+    s << "#  endif // " << innerHeaderGuard << "\n\n";
 }
 
 void HeaderGenerator::writeWrapperClassDeclaration(TextStream &s,
                                                    const QString &wrapperName,
                                                    const GeneratorContext &classContext) const
 {
-    const AbstractMetaClassCPtr metaClass = classContext.metaClass();
+    const AbstractMetaClassCPtr &metaClass = classContext.metaClass();
     const auto typeEntry = metaClass->typeEntry();
     InheritedOverloadSet inheritedOverloads;
 
