@@ -1,31 +1,32 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
+#include "autodecref.h"
 #include "basewrapper.h"
 #include "basewrapper_p.h"
 #include "bindingmanager.h"
+#include "gilstate.h"
 #include "helper.h"
+#include "pep384ext.h"
 #include "sbkconverter.h"
-#include "sbkenum.h"
 #include "sbkerrors.h"
 #include "sbkfeature_base.h"
-#include "sbkstring.h"
 #include "sbkstaticstrings.h"
 #include "sbkstaticstrings_p.h"
-#include "autodecref.h"
-#include "gilstate.h"
-#include <string>
-#include <cstring>
-#include <cstddef>
-#include <set>
-#include <sstream>
-#include <algorithm>
-#include "threadstatesaver.h"
+#include "sbkstring.h"
+#include "sbktypefactory.h"
 #include "signature.h"
 #include "signature_p.h"
+#include "threadstatesaver.h"
 #include "voidptr.h"
 
+#include <algorithm>
+#include <cstddef>
+#include <cstring>
 #include <iostream>
+#include <set>
+#include <sstream>
+#include <string>
 
 #if defined(__APPLE__)
 #include <dlfcn.h>
@@ -35,7 +36,73 @@ namespace {
     void _destroyParentInfo(SbkObject *obj, bool keepReference);
 }
 
-static void callDestructor(const Shiboken::DtorAccumulatorVisitor::DestructorEntries &dts)
+namespace Shiboken
+{
+// Walk through the first level of non-user-type Sbk base classes relevant for
+// C++ object allocation. Return true from the predicate to terminate.
+template <class Predicate>
+bool walkThroughBases(PyTypeObject *currentType, Predicate predicate)
+{
+    PyObject *bases = currentType->tp_bases;
+    const Py_ssize_t numBases = PyTuple_Size(bases);
+    bool result = false;
+    for (Py_ssize_t i = 0; !result && i < numBases; ++i) {
+        auto *type = reinterpret_cast<PyTypeObject *>(PyTuple_GetItem(bases, i));
+        if (PyType_IsSubtype(type, SbkObject_TypeF()) != 0) {
+            result = PepType_SOTP(type)->is_user_type
+                     ? walkThroughBases(type, predicate) : predicate(type);
+        }
+    }
+    return result;
+}
+
+int getTypeIndexOnHierarchy(PyTypeObject *baseType, PyTypeObject *desiredType)
+{
+    int index = -1;
+    walkThroughBases(baseType, [&index, desiredType](PyTypeObject *node) {
+        ++index;
+        return PyType_IsSubtype(node, desiredType) != 0;
+    });
+    return index;
+}
+
+int getNumberOfCppBaseClasses(PyTypeObject *baseType)
+{
+    int count = 0;
+    walkThroughBases(baseType, [&count](PyTypeObject *) {
+        ++count;
+        return false;
+    });
+    return count;
+}
+
+std::vector<PyTypeObject *> getCppBaseClasses(PyTypeObject *baseType)
+{
+    std::vector<PyTypeObject *> cppBaseClasses;
+    walkThroughBases(baseType, [&cppBaseClasses](PyTypeObject *node) {
+        cppBaseClasses.push_back(node);
+        return false;
+    });
+    return cppBaseClasses;
+}
+
+using DestructorEntries = std::vector<DestructorEntry>;
+
+DestructorEntries getDestructorEntries(SbkObject *o)
+{
+    DestructorEntries result;
+    void **cptrs = o->d->cptr;
+    walkThroughBases(Py_TYPE(o), [&result, cptrs](PyTypeObject *node) {
+        auto *sotp = PepType_SOTP(node);
+        auto index = result.size();
+        result.push_back(DestructorEntry{sotp->cpp_dtor,
+                                         cptrs[index]});
+        return false;
+    });
+    return result;
+}
+
+static void callDestructor(const DestructorEntries &dts)
 {
     for (const auto &e : dts) {
         Shiboken::ThreadStateSaver threadSaver;
@@ -44,18 +111,19 @@ static void callDestructor(const Shiboken::DtorAccumulatorVisitor::DestructorEnt
     }
 }
 
+} // namespace Shiboken
+
 extern "C"
 {
 
 // PYSIDE-939: A general replacement for object_dealloc.
 void Sbk_object_dealloc(PyObject *self)
 {
-    if (PepRuntime_38_flag) {
-        // PYSIDE-939: Handling references correctly.
-        // This was not needed before Python 3.8 (Python issue 35810)
-        Py_DECREF(Py_TYPE(self));
-    }
-    Py_TYPE(self)->tp_free(self);
+    // PYSIDE-939: Handling references correctly.
+    // This was not needed before Python 3.8 (Python issue 35810)
+    Py_DECREF(Py_TYPE(self));
+
+    PepExt_TypeCallFree(self);
 }
 
 static void SbkObjectType_tp_dealloc(PyTypeObject *pyType);
@@ -152,11 +220,21 @@ static PyTypeObject *createObjectTypeType()
         "1:Shiboken.ObjectType",
         static_cast<int>(PyType_Type.tp_basicsize) + 1,           // see above
         0, // sizeof(PyMemberDef), not for PyPy without a __len__ defined
-        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_TYPE_SUBCLASS,
         SbkObjectType_Type_slots,
     };
 
-    return SbkType_FromSpec(&SbkObjectType_Type_spec);
+    PyType_Spec SbkObjectType_Type_spec_312 = {
+        "1:Shiboken.ObjectType",
+        -long(sizeof(SbkObjectTypePrivate)),
+        0, // sizeof(PyMemberDef), not for PyPy without a __len__ defined
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_TYPE_SUBCLASS,
+        SbkObjectType_Type_slots,
+    };
+
+    return SbkType_FromSpec(_PepRuntimeVersion() >= 0x030C00 ?
+                            &SbkObjectType_Type_spec_312 :
+                            &SbkObjectType_Type_spec);
 }
 
 PyTypeObject *SbkObjectType_TypeF(void)
@@ -167,7 +245,7 @@ PyTypeObject *SbkObjectType_TypeF(void)
 
 static PyObject *SbkObjectGetDict(PyObject *pObj, void *)
 {
-    auto ret = SbkObject_GetDict_NoRef(pObj);
+    auto *ret = SbkObject_GetDict_NoRef(pObj);
     Py_XINCREF(ret);
     return ret;
 }
@@ -182,26 +260,22 @@ static int SbkObject_tp_traverse(PyObject *self, visitproc visit, void *arg)
     auto *sbkSelf = reinterpret_cast<SbkObject *>(self);
 
     //Visit children
-    Shiboken::ParentInfo *pInfo = sbkSelf->d->parentInfo;
-    if (pInfo) {
+    if (auto *pInfo = sbkSelf->d->parentInfo) {
         for (SbkObject *c : pInfo->children)
              Py_VISIT(c);
     }
 
     //Visit refs
-    Shiboken::RefCountMap *rInfo = sbkSelf->d->referredObjects;
-    if (rInfo) {
-        for (auto it = rInfo->begin(), end = rInfo->end(); it != end; ++it)
-            Py_VISIT(it->second);
+    if (auto *rInfo = sbkSelf->d->referredObjects) {
+        for (const auto &p : *rInfo)
+            Py_VISIT(p.second);
     }
 
     if (sbkSelf->ob_dict)
         Py_VISIT(sbkSelf->ob_dict);
 
-#if PY_VERSION_HEX >= 0x03090000
     // This was not needed before Python 3.9 (Python issue 35810 and 40217)
     Py_VISIT(Py_TYPE(self));
-#endif
     return 0;
 }
 
@@ -249,12 +323,13 @@ static PyTypeObject *createObjectType()
     //              But before 3.12 is the minimum version, we cannot use the new
     //              function, although we would need this for 3.12 :-D
     //              We do a special patching here that is triggered through Py_None.
-    return SbkType_FromSpec_BMDWB(&SbkObject_Type_spec,
-                                  Py_None,     // bases, special flag!
-                                  SbkObjectType_TypeF(),
-                                  offsetof(SbkObject, ob_dict),
-                                  offsetof(SbkObject, weakreflist),
-                                  nullptr);    // bufferprocs
+    auto *type = SbkType_FromSpec_BMDWB(&SbkObject_Type_spec,
+                                        Py_None,     // bases, spectial flag!
+                                        SbkObjectType_TypeF(),
+                                        offsetof(SbkObject, ob_dict),
+                                        offsetof(SbkObject, weakreflist),
+                                        nullptr);    // bufferprocs
+    return type;
 }
 
 PyTypeObject *SbkObject_TypeF(void)
@@ -282,15 +357,14 @@ static void SbkDeallocWrapperCommon(PyObject *pyObj, bool canDelete)
     // Need to decref the type if this is the dealloc func; if type
     // is subclassed, that dealloc func will decref (see subtype_dealloc
     // in typeobject.c in the python sources)
-    auto dealloc = PyType_GetSlot(pyType, Py_tp_dealloc);
-    bool needTypeDecref = dealloc == SbkDeallocWrapper
-        || dealloc == SbkDeallocWrapperWithPrivateDtor;
-    if (PepRuntime_38_flag) {
-        // PYSIDE-939: Additional rule: Also when a subtype is heap allocated,
-        // then the subtype_dealloc deref will be suppressed, and we need again
-        // to supply a decref.
-        needTypeDecref |= (pyType->tp_base->tp_flags & Py_TPFLAGS_HEAPTYPE) != 0;
-    }
+    auto *dealloc = PyType_GetSlot(pyType, Py_tp_dealloc);
+
+    // PYSIDE-939: Additional rule: Also when a subtype is heap allocated,
+    // then the subtype_dealloc deref will be suppressed, and we need again
+    // to supply a decref.
+    const bool needTypeDecref = dealloc == SbkDeallocWrapper
+        || dealloc == SbkDeallocWrapperWithPrivateDtor
+        || (pyType->tp_base->tp_flags & Py_TPFLAGS_HEAPTYPE) != 0;
 
 #if defined(__APPLE__)
     // Just checking once that our assumptions are right.
@@ -329,9 +403,8 @@ static void SbkDeallocWrapperCommon(PyObject *pyObj, bool canDelete)
         if (sotp->delete_in_main_thread && Shiboken::currentThreadId() != Shiboken::mainThreadId()) {
             auto &bindingManager = Shiboken::BindingManager::instance();
             if (sotp->is_multicpp) {
-                 Shiboken::DtorAccumulatorVisitor visitor(sbkObj);
-                 Shiboken::walkThroughClassHierarchy(Py_TYPE(pyObj), &visitor);
-                 for (const auto &e : visitor.entries())
+                 const auto entries = Shiboken::getDestructorEntries(sbkObj);
+                 for (const auto &e : entries)
                      bindingManager.addToDeletionInMainThread(e);
             } else {
                 Shiboken::DestructorEntry e{sotp->cpp_dtor, sbkObj->d->cptr[0]};
@@ -342,17 +415,18 @@ static void SbkDeallocWrapperCommon(PyObject *pyObj, bool canDelete)
         }
     }
 
-    PyObject *error_type, *error_value, *error_traceback;
+    PyObject *error_type{};
+    PyObject *error_value{};
+    PyObject *error_traceback{};
 
     /* Save the current exception, if any. */
     PyErr_Fetch(&error_type, &error_value, &error_traceback);
 
     if (canDelete) {
         if (sotp->is_multicpp) {
-            Shiboken::DtorAccumulatorVisitor visitor(sbkObj);
-            Shiboken::walkThroughClassHierarchy(Py_TYPE(pyObj), &visitor);
+            const auto entries = Shiboken::getDestructorEntries(sbkObj);
             Shiboken::Object::deallocData(sbkObj, true);
-            callDestructor(visitor.entries());
+            callDestructor(entries);
         } else {
             void *cptr = sbkObj->d->cptr[0];
             Shiboken::Object::deallocData(sbkObj, true);
@@ -371,11 +445,9 @@ static void SbkDeallocWrapperCommon(PyObject *pyObj, bool canDelete)
 
     if (needTypeDecref)
         Py_DECREF(pyType);
-    if (PepRuntime_38_flag) {
-        // PYSIDE-939: Handling references correctly.
-        // This was not needed before Python 3.8 (Python issue 35810)
-        Py_DECREF(pyType);
-    }
+    // PYSIDE-939: Handling references correctly.
+    // This was not needed before Python 3.8 (Python issue 35810)
+    Py_DECREF(pyType);
 }
 
 static inline PyObject *_Sbk_NewVarObject(PyTypeObject *type)
@@ -409,7 +481,7 @@ void SbkDeallocWrapperWithPrivateDtor(PyObject *self)
 void SbkObjectType_tp_dealloc(PyTypeObject *sbkType)
 {
     SbkObjectTypePrivate *sotp = PepType_SOTP(sbkType);
-    auto pyObj = reinterpret_cast<PyObject *>(sbkType);
+    auto *pyObj = reinterpret_cast<PyObject *>(sbkType);
 
     PyObject_GC_UnTrack(pyObj);
 #if !defined(Py_LIMITED_API) && !defined(PYPY_VERSION)
@@ -437,11 +509,9 @@ void SbkObjectType_tp_dealloc(PyTypeObject *sbkType)
     Py_TRASHCAN_SAFE_END(pyObj);
 #  endif
 #endif
-    if (PepRuntime_38_flag) {
-        // PYSIDE-939: Handling references correctly.
-        // This was not needed before Python 3.8 (Python issue 35810)
-        Py_DECREF(Py_TYPE(pyObj));
-    }
+    // PYSIDE-939: Handling references correctly.
+    // This was not needed before Python 3.8 (Python issue 35810)
+    Py_DECREF(Py_TYPE(pyObj));
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -458,7 +528,7 @@ PyObject *MakeQAppWrapper(PyTypeObject *type)
     static PyObject *qApp_last = nullptr;
 
     // protecting from multiple application instances
-    if (!(type == nullptr || qApp_last == Py_None)) {
+    if (type != nullptr && qApp_last != Py_None) {
         const char *res_name = qApp_last != nullptr
             ? PepType_GetNameStr(Py_TYPE(qApp_last)) : "<Unknown>";
         const char *type_name = PepType_GetNameStr(type);
@@ -496,9 +566,9 @@ static PyTypeObject *SbkObjectType_tp_new(PyTypeObject *metatype, PyObject *args
     // Before we changed to heap types, it was sufficient to remove the
     // Py_TPFLAGS_BASETYPE flag. That does not work, because PySide does
     // not respect this flag itself!
-    PyObject *name;
-    PyObject *pyBases;
-    PyObject *dict;
+    PyObject *name{};
+    PyObject *pyBases{};
+    PyObject *dict{};
     static const char *kwlist[] = { "name", "bases", "dict", nullptr};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO!O!:sbktype", const_cast<char **>(kwlist),
@@ -507,9 +577,9 @@ static PyTypeObject *SbkObjectType_tp_new(PyTypeObject *metatype, PyObject *args
                                      &PyDict_Type, &dict))
         return nullptr;
 
-    for (int i=0, i_max=PyTuple_GET_SIZE(pyBases); i < i_max; i++) {
-        PyObject *baseType = PyTuple_GET_ITEM(pyBases, i);
-        if (reinterpret_cast<PyTypeObject *>(baseType)->tp_new == SbkDummyNew) {
+    for (Py_ssize_t i=0, i_max=PyTuple_Size(pyBases); i < i_max; i++) {
+        PyObject *baseType = PyTuple_GetItem(pyBases, i);
+        if (PepExt_Type_GetNewSlot(reinterpret_cast<PyTypeObject *>(baseType)) == SbkDummyNew) {
             // PYSIDE-595: A base class does not allow inheritance.
             return reinterpret_cast<PyTypeObject *>(SbkDummyNew(metatype, args, kwds));
         }
@@ -584,7 +654,7 @@ static PyObject *_setupNew(PyObject *obSelf, PyTypeObject *subtype)
     auto *self = reinterpret_cast<SbkObject *>(obSelf);
 
     Py_INCREF(obSubtype);
-    auto d = new SbkObjectPrivate;
+    auto *d = new SbkObjectPrivate;
 
     auto *sotp = PepType_SOTP(sbkSubtype);
     int numBases = ((sotp && sotp->is_multicpp) ?
@@ -617,9 +687,8 @@ PyObject *SbkQApp_tp_new(PyTypeObject *subtype, PyObject *, PyObject *)
     auto *self = reinterpret_cast<SbkObject *>(obSelf);
     if (self == nullptr)
         return nullptr;
-    auto ret = _setupNew(obSelf, subtype);
-    auto priv = self->d;
-    priv->isQAppSingleton = 1;
+    auto *ret = _setupNew(obSelf, subtype);
+    self->d->isQAppSingleton = 1;
     return ret;
 }
 
@@ -637,7 +706,7 @@ PyObject *FallbackRichCompare(PyObject *self, PyObject *other, int op)
 {
     // This is a very simple implementation that supplies a simple identity.
     static const char * const opstrings[] = {"<", "<=", "==", "!=", ">", ">="};
-    PyObject *res;
+    PyObject *res{};
 
     switch (op) {
 
@@ -661,10 +730,57 @@ PyObject *FallbackRichCompare(PyObject *self, PyObject *other, int op)
 
 bool SbkObjectType_Check(PyTypeObject *type)
 {
-    static auto *obMeta = reinterpret_cast<PyObject *>(SbkObjectType_TypeF());
-    auto *obType = reinterpret_cast<PyObject *>(type);
-    return obMeta == reinterpret_cast<PyObject *>(Py_TYPE(obType))
-           || PyObject_IsInstance(obType, obMeta);
+    static auto *meta = SbkObjectType_TypeF();
+    return Py_TYPE(type) == meta || PyType_IsSubtype(Py_TYPE(type), meta);
+}
+
+// Global functions from folding.
+
+// The common end.
+PyObject *Sbk_ReturnFromPython_None()
+{
+    if (Shiboken::Errors::occurred() != nullptr) {
+        return {};
+    }
+    Py_RETURN_NONE;
+}
+
+PyObject *Sbk_ReturnFromPython_Result(PyObject *pyResult)
+{
+    if (Shiboken::Errors::occurred() != nullptr || pyResult == nullptr) {
+        Py_XDECREF(pyResult);
+        return {};
+    }
+    return pyResult;
+}
+
+PyObject *Sbk_ReturnFromPython_Self(PyObject *self)
+{
+    if (Shiboken::Errors::occurred() != nullptr) {
+        return {};
+    }
+    Py_INCREF(self);
+    return self;
+}
+
+// The virtual function call
+PyObject *Sbk_GetPyOverride(const void *voidThis, Shiboken::GilState &gil, const char *funcName,
+                            bool *resultCache, PyObject **nameCache)
+{
+    PyObject *pyOverride{};
+    if (!*resultCache) {
+        gil.acquire();
+        pyOverride = Shiboken::BindingManager::instance().getOverride(voidThis, nameCache, funcName);
+        if (pyOverride == nullptr) {
+            *resultCache = true;
+            gil.release();
+        } else if (Shiboken::Errors::occurred() != nullptr) {
+            // Give up.
+            Py_XDECREF(pyOverride);
+            pyOverride = nullptr;
+        }
+    }
+    return pyOverride;
 }
 
 } //extern "C"
@@ -691,54 +807,24 @@ void _destroyParentInfo(SbkObject *obj, bool keepReference)
 
 namespace Shiboken
 {
-bool walkThroughClassHierarchy(PyTypeObject *currentType, HierarchyVisitor *visitor)
-{
-    PyObject *bases = currentType->tp_bases;
-    Py_ssize_t numBases = PyTuple_GET_SIZE(bases);
-    bool result = false;
-    for (int i = 0; !result && i < numBases; ++i) {
-        auto type = reinterpret_cast<PyTypeObject *>(PyTuple_GET_ITEM(bases, i));
-        if (PyType_IsSubtype(type, reinterpret_cast<PyTypeObject *>(SbkObject_TypeF()))) {
-            result = PepType_SOTP(type)->is_user_type
-                     ? walkThroughClassHierarchy(type, visitor) : visitor->visit(type);
-        }
-    }
-    return result;
-}
 
 // Wrapper metatype and base type ----------------------------------------------------------
 
-HierarchyVisitor::HierarchyVisitor() = default;
-HierarchyVisitor::~HierarchyVisitor() = default;
-
-bool BaseCountVisitor::visit(PyTypeObject *)
-{
-    m_count++;
-    return false;
-}
-
-bool BaseAccumulatorVisitor::visit(PyTypeObject *node)
-{
-    m_bases.push_back(node);
-    return false;
-}
-
-bool GetIndexVisitor::visit(PyTypeObject *node)
-{
-    m_index++;
-    return PyType_IsSubtype(node, m_desiredType);
-}
-
-bool DtorAccumulatorVisitor::visit(PyTypeObject *node)
-{
-    auto *sotp = PepType_SOTP(node);
-    m_entries.push_back(DestructorEntry{sotp->cpp_dtor,
-                                        m_pyObject->d->cptr[m_entries.size()]});
-    return false;
-}
-
 void _initMainThreadId(); // helper.cpp
 
+static std::string msgFailedToInitializeType(const char *description)
+{
+    std::ostringstream stream;
+    stream << "[libshiboken] Failed to initialize " << description;
+    if (auto *error = PepErr_GetRaisedException()) {
+        if (auto *str = PyObject_Str(error))
+            stream << ": " << Shiboken::String::toCString(str);
+        Py_DECREF(error);
+    }
+    stream << '.';
+    return stream.str();
+}
+
 namespace Conversions { void init(); }
 
 void init()
@@ -754,11 +840,13 @@ void init()
     //Init private data
     Pep384_Init();
 
-    if (PyType_Ready(SbkObjectType_TypeF()) < 0)
-        Py_FatalError("[libshiboken] Failed to initialize Shiboken.BaseWrapperType metatype.");
+    auto *type = SbkObjectType_TypeF();
+    if (type == nullptr || PyType_Ready(type) < 0)
+        Py_FatalError(msgFailedToInitializeType("Shiboken.BaseWrapperType metatype").c_str());
 
-    if (PyType_Ready(SbkObject_TypeF()) < 0)
-        Py_FatalError("[libshiboken] Failed to initialize Shiboken.BaseWrapper type.");
+    type = SbkObject_TypeF();
+    if (type == nullptr || PyType_Ready(type) < 0)
+        Py_FatalError(msgFailedToInitializeType("Shiboken.BaseWrapper type").c_str());
 
     VoidPtr::init();
 
@@ -784,26 +872,40 @@ void initShibokenSupport(PyObject *module)
 
 // setErrorAboutWrongArguments now gets overload info from the signature module.
 // Info can be nullptr and contains extra info.
-void setErrorAboutWrongArguments(PyObject *args, const char *funcName, PyObject *info)
+void setErrorAboutWrongArguments(PyObject *args, const char *funcName, PyObject *info,
+                                 const char *className)
 {
+    if (className != nullptr) {
+        std::string text = std::string(className);
+        text += '.';
+        text += funcName;
+        SetError_Argument(args, text.c_str(), info);
+        return;
+    }
     SetError_Argument(args, funcName, info);
 }
 
-PyObject *returnWrongArguments(PyObject *args, const char *funcName, PyObject *info)
+PyObject *returnWrongArguments(PyObject *args, const char *funcName, PyObject *info,
+                               Module::TypeInitStruct initStruct)
 {
-    setErrorAboutWrongArguments(args, funcName, info);
+    const auto *className = initStruct.fullName ? initStruct.fullName : nullptr;
+    setErrorAboutWrongArguments(args, funcName, info, className);
     return {};
 }
 
-int returnWrongArguments_Zero(PyObject *args, const char *funcName, PyObject *info)
+int returnWrongArguments_Zero(PyObject *args, const char *funcName, PyObject *info,
+                              Module::TypeInitStruct initStruct)
 {
-    setErrorAboutWrongArguments(args, funcName, info);
+    const auto *className = initStruct.fullName ? initStruct.fullName : nullptr;
+    setErrorAboutWrongArguments(args, funcName, info, className);
     return 0;
 }
 
-int returnWrongArguments_MinusOne(PyObject *args, const char *funcName, PyObject *info)
+int returnWrongArguments_MinusOne(PyObject *args, const char *funcName, PyObject *info,
+                                  Module::TypeInitStruct initStruct)
 {
-    setErrorAboutWrongArguments(args, funcName, info);
+    const auto *className = initStruct.fullName ? initStruct.fullName : nullptr;
+    setErrorAboutWrongArguments(args, funcName, info, className);
     return -1;
 }
 
@@ -832,30 +934,16 @@ PyObject *checkInvalidArgumentCount(Py_ssize_t numArgs, Py_ssize_t minArgs, Py_s
     return result;
 }
 
-class FindBaseTypeVisitor : public HierarchyVisitor
-{
-public:
-    explicit FindBaseTypeVisitor(PyTypeObject *typeToFind) : m_typeToFind(typeToFind) {}
-
-    bool visit(PyTypeObject *node) override
-    {
-        return node == m_typeToFind;
-    }
-
-private:
-    PyTypeObject *m_typeToFind;
-};
-
 std::vector<SbkObject *> splitPyObject(PyObject *pyObj)
 {
     std::vector<SbkObject *> result;
     if (PySequence_Check(pyObj)) {
         AutoDecRef lst(PySequence_Fast(pyObj, "Invalid keep reference object."));
         if (!lst.isNull()) {
-            for (Py_ssize_t i = 0, i_max = PySequence_Fast_GET_SIZE(lst.object()); i < i_max; ++i) {
-                PyObject *item = PySequence_Fast_GET_ITEM(lst.object(), i);
+            for (Py_ssize_t i = 0, i_max = PySequence_Size(lst.object()); i < i_max; ++i) {
+                Shiboken::AutoDecRef item(PySequence_GetItem(lst.object(), i));
                 if (Object::checkType(item))
-                    result.push_back(reinterpret_cast<SbkObject *>(item));
+                    result.push_back(reinterpret_cast<SbkObject *>(item.object()));
             }
         }
     } else {
@@ -886,8 +974,8 @@ bool isUserType(PyTypeObject *type)
 
 bool canCallConstructor(PyTypeObject *myType, PyTypeObject *ctorType)
 {
-    FindBaseTypeVisitor visitor(ctorType);
-    if (!walkThroughClassHierarchy(myType, &visitor)) {
+    auto findBasePred = [ctorType](PyTypeObject *type) { return type == ctorType; };
+    if (!walkThroughBases(myType, findBasePred)) {
         PyErr_Format(PyExc_TypeError, "%s isn't a direct base class of %s", ctorType->tp_name, myType->tp_name);
         return false;
     }
@@ -959,24 +1047,19 @@ introduceWrapperType(PyObject *enclosingObject,
                      const char *originalName,
                      PyType_Spec *typeSpec,
                      ObjectDestructor cppObjDtor,
-                     PyTypeObject *baseType,
-                     PyObject *baseTypes,
+                     PyObject *bases,
                      unsigned wrapperFlags)
 {
-    auto *base = baseType ? baseType : SbkObject_TypeF();
-    typeSpec->slots[0].pfunc = reinterpret_cast<void *>(base);
-    auto *bases = baseTypes ? baseTypes : PyTuple_Pack(1, base);
+    assert(PySequence_Size(bases) > 0);
+    typeSpec->slots[0].pfunc = PySequence_GetItem(bases, 0);
 
     auto *type = SbkType_FromSpecBasesMeta(typeSpec, bases, SbkObjectType_TypeF());
 
-    for (int i = 0; i < PySequence_Fast_GET_SIZE(bases); ++i) {
-        auto *st = reinterpret_cast<PyTypeObject *>(PySequence_Fast_GET_ITEM(bases, i));
-        BindingManager::instance().addClassInheritance(st, type);
-    }
-
-    auto sotp = PepType_SOTP(type);
+    auto *sotp = PepType_SOTP(type);
     if (wrapperFlags & DeleteInMainThread)
         sotp->delete_in_main_thread = 1;
+    sotp->type_behaviour = (wrapperFlags & Value) != 0
+                           ? BEHAVIOUR_VALUETYPE : BEHAVIOUR_OBJECTTYPE;
 
     setOriginalName(type, originalName);
     setDestructorFunction(type, cppObjDtor);
@@ -989,8 +1072,8 @@ introduceWrapperType(PyObject *enclosingObject,
             AutoDecRef tpDict(PepType_GetDict(reinterpret_cast<PyTypeObject *>(enclosingObject)));
             return PyDict_SetItemString(tpDict, typeName, ob_type) == 0 ? type : nullptr;
         }
-        assert(PyDict_Check(enclosingObject));
-        return PyDict_SetItemString(enclosingObject, typeName, ob_type) == 0 ? type : nullptr;
+        if (PyDict_Check(enclosingObject))
+            return PyDict_SetItemString(enclosingObject, typeName, ob_type) == 0 ? type : nullptr;
     }
 
     // PyModule_AddObject steals type's reference.
@@ -1006,16 +1089,19 @@ introduceWrapperType(PyObject *enclosingObject,
 
 void setSubTypeInitHook(PyTypeObject *type, SubTypeInitHook func)
 {
+    assert(SbkObjectType_Check(type));
     PepType_SOTP(type)->subtype_init = func;
 }
 
 void *getTypeUserData(PyTypeObject *type)
 {
+    assert(SbkObjectType_Check(type));
     return PepType_SOTP(type)->user_data;
 }
 
 void setTypeUserData(PyTypeObject *type, void *userData, DeleteUserDataFunc d_func)
 {
+    assert(SbkObjectType_Check(type));
     auto *sotp = PepType_SOTP(type);
     sotp->user_data = userData;
     sotp->d_func = d_func;
@@ -1038,13 +1124,33 @@ bool hasSpecialCastFunction(PyTypeObject *sbkType)
     return d != nullptr && d->mi_specialcast != nullptr;
 }
 
+// Find whether base is a direct single line base class of type
+// (no multiple inheritance), that is, a C++ pointer cast can safely be done.
+static bool isDirectAncestor(PyTypeObject *type, PyTypeObject *base)
+{
+    if (type == base)
+        return true;
+    if (PyTuple_Size(type->tp_bases) == 0)
+        return false;
+    auto *sbkObjectType = SbkObject_TypeF();
+    auto *firstBase = reinterpret_cast<PyTypeObject *>(PyTuple_GetItem(type->tp_bases, 0));
+    return firstBase != sbkObjectType
+           && PyType_IsSubtype(type, sbkObjectType) != 0
+           && isDirectAncestor(firstBase, base);
+}
+
+bool canDowncastTo(PyTypeObject *baseType, PyTypeObject *targetType)
+{
+    return isDirectAncestor(targetType, baseType);
+}
+
 } // namespace ObjectType
 
 
 namespace Object
 {
 
-static void recursive_invalidate(SbkObject *self, std::set<SbkObject *>& seen);
+static void recursive_invalidate(SbkObject *self, std::set<SbkObject *> &seen);
 
 bool checkType(PyObject *pyObj)
 {
@@ -1117,7 +1223,7 @@ bool wasCreatedByPython(SbkObject *pyObj)
 
 void callCppDestructors(SbkObject *pyObj)
 {
-    auto priv = pyObj->d;
+    auto *priv = pyObj->d;
     if (priv->isQAppSingleton && DestroyQApplication) {
         // PYSIDE-1470: Allow to destroy the application from Shiboken.
         DestroyQApplication();
@@ -1126,9 +1232,7 @@ void callCppDestructors(SbkObject *pyObj)
     PyTypeObject *type = Py_TYPE(pyObj);
     auto *sotp = PepType_SOTP(type);
     if (sotp->is_multicpp) {
-        Shiboken::DtorAccumulatorVisitor visitor(pyObj);
-        Shiboken::walkThroughClassHierarchy(type, &visitor);
-        callDestructor(visitor.entries());
+        callDestructor(getDestructorEntries(pyObj));
     } else {
         Shiboken::ThreadStateSaver threadSaver;
         threadSaver.save();
@@ -1203,8 +1307,7 @@ void releaseOwnership(PyObject *self)
 }
 
 /* Needed forward declarations */
-static void recursive_invalidate(PyObject *pyobj, std::set<SbkObject *>& seen);
-static void recursive_invalidate(SbkObject *self, std::set<SbkObject *> &seen);
+static void recursive_invalidate(PyObject *pyobj, std::set<SbkObject *> &seen);
 
 void invalidate(PyObject *pyobj)
 {
@@ -1253,10 +1356,9 @@ static void recursive_invalidate(SbkObject *self, std::set<SbkObject *> &seen)
     }
 
     // If has ref to other objects invalidate all
-    if (self->d->referredObjects) {
-        RefCountMap &refCountMap = *(self->d->referredObjects);
-        for (auto it = refCountMap.begin(), end = refCountMap.end(); it != end; ++it)
-            recursive_invalidate(it->second, seen);
+    if (auto *rInfo = self->d->referredObjects) {
+        for (const auto &p : *rInfo)
+            recursive_invalidate(p.second, seen);
     }
 }
 
@@ -1409,28 +1511,74 @@ SbkObject *findColocatedChild(SbkObject *wrapper,
     return nullptr;
 }
 
+// Legacy, for compatibility only.
 PyObject *newObject(PyTypeObject *instanceType,
                     void *cptr,
                     bool hasOwnership,
                     bool isExactType,
                     const char *typeName)
 {
-    // Try to find the exact type of cptr.
-    if (!isExactType) {
-        if (PyTypeObject *exactType = ObjectType::typeForTypeName(typeName))
-            instanceType = exactType;
-        else
-            instanceType = BindingManager::instance().resolveType(&cptr, instanceType);
+    return isExactType
+        ? newObjectForType(instanceType, cptr, hasOwnership)
+        : newObjectWithHeuristics(instanceType, cptr, hasOwnership, typeName);
+}
+
+static PyObject *newObjectWithHeuristicsHelper(PyTypeObject *instanceType,
+                                               PyTypeObject *exactType,
+                                               void *cptr,
+                                               bool hasOwnership)
+{
+    // Try to find the exact type of cptr. For hierarchies with
+    // non-virtual destructors, typeid() will return the base name.
+    // Try type discovery in these cases.
+    if (exactType == nullptr || exactType == instanceType) {
+        auto resolved = BindingManager::instance().findDerivedType(cptr, instanceType);
+        if (resolved.first != nullptr
+            && Shiboken::ObjectType::canDowncastTo(instanceType, resolved.first)) {
+            exactType = resolved.first;
+            cptr = resolved.second;
+        }
     }
 
+    return newObjectForType(exactType != nullptr ? exactType : instanceType,
+                            cptr, hasOwnership);
+}
+
+PyObject *newObjectForPointer(PyTypeObject *instanceType,
+                              void *cptr,
+                              bool hasOwnership,
+                              const char *typeName)
+{
+    // Try to find the exact type of cptr.
+    PyTypeObject *exactType = ObjectType::typeForTypeName(typeName);
+    // PYSIDE-868: In case of multiple inheritance, (for example,
+    // a function returning a QPaintDevice * from a QWidget *),
+    // use instance type to avoid pointer offset errors.
+    return exactType != nullptr && !Shiboken::ObjectType::canDowncastTo(instanceType, exactType)
+        ? newObjectForType(instanceType, cptr, hasOwnership)
+        : newObjectWithHeuristicsHelper(instanceType, exactType, cptr, hasOwnership);
+}
+
+
+PyObject *newObjectWithHeuristics(PyTypeObject *instanceType,
+                                  void *cptr,
+                                  bool hasOwnership,
+                                  const char *typeName)
+{
+    return newObjectWithHeuristicsHelper(instanceType,
+                                         ObjectType::typeForTypeName(typeName),
+                                         cptr, hasOwnership);
+}
+
+PyObject *newObjectForType(PyTypeObject *instanceType, void *cptr, bool hasOwnership)
+{
     bool shouldCreate = true;
     bool shouldRegister = true;
     SbkObject *self = nullptr;
 
+    auto &bindingManager = BindingManager::instance();
     // Some logic to ensure that colocated child field does not overwrite the parent
-    if (BindingManager::instance().hasWrapper(cptr)) {
-        SbkObject *existingWrapper = BindingManager::instance().retrieveWrapper(cptr);
-
+    if (SbkObject *existingWrapper = bindingManager.retrieveWrapper(cptr)) {
         self = findColocatedChild(existingWrapper, instanceType);
         if (self) {
             // Wrapper already registered for cptr.
@@ -1441,7 +1589,7 @@ PyObject *newObject(PyTypeObject *instanceType,
                   (!(Shiboken::Object::hasCppWrapper(existingWrapper) ||
                      Shiboken::Object::hasOwnership(existingWrapper)))) {
             // Old wrapper is likely junk, since we have ownership and it doesn't.
-            BindingManager::instance().releaseWrapper(existingWrapper);
+            bindingManager.releaseWrapper(existingWrapper);
         } else {
             // Old wrapper may be junk caused by some bug in identifying object deletion
             // but it may not be junk when a colocated field is accessed for an
@@ -1456,9 +1604,8 @@ PyObject *newObject(PyTypeObject *instanceType,
         self->d->cptr[0] = cptr;
         self->d->hasOwnership = hasOwnership;
         self->d->validCppObject = 1;
-        if (shouldRegister) {
-            BindingManager::instance().registerWrapper(self, cptr);
-        }
+        if (shouldRegister)
+            bindingManager.registerWrapper(self, cptr);
     } else {
         Py_IncRef(reinterpret_cast<PyObject *>(self));
     }
@@ -1562,14 +1709,16 @@ void setParent(PyObject *parent, PyObject *child)
      */
     if (PySequence_Check(child) && !Object::checkType(child)) {
         Shiboken::AutoDecRef seq(PySequence_Fast(child, nullptr));
-        for (Py_ssize_t i = 0, max = PySequence_Size(seq); i < max; ++i)
-            setParent(parent, PySequence_Fast_GET_ITEM(seq.object(), i));
+        for (Py_ssize_t i = 0, max = PySequence_Size(seq); i < max; ++i) {
+            Shiboken::AutoDecRef obj(PySequence_GetItem(seq.object(), i));
+            setParent(parent, obj);
+        }
         return;
     }
 
     bool parentIsNull = !parent || parent == Py_None;
-    auto parent_ = reinterpret_cast<SbkObject *>(parent);
-    auto child_ = reinterpret_cast<SbkObject *>(child);
+    auto *parent_ = reinterpret_cast<SbkObject *>(parent);
+    auto *child_ = reinterpret_cast<SbkObject *>(child);
 
     if (!parentIsNull) {
         if (!parent_->d->parentInfo)
@@ -1631,7 +1780,7 @@ void deallocData(SbkObject *self, bool cleanup)
     }
     delete self->d; // PYSIDE-205: always delete d.
     Py_XDECREF(self->ob_dict);
-    Py_TYPE(self)->tp_free(self);
+    PepExt_TypeCallFree(reinterpret_cast<PyObject *>(self));
 }
 
 void setTypeUserData(SbkObject *wrapper, void *userData, DeleteUserDataFunc d_func)
@@ -1656,7 +1805,7 @@ static inline bool isNone(const PyObject *o)
     return o == nullptr || o == Py_None;
 }
 
-static void removeRefCountKey(SbkObject *self, const char *key)
+static void removeRefCountKey(SbkObject *self, const std::string &key)
 {
     if (self->d->referredObjects) {
         const auto iterPair = self->d->referredObjects->equal_range(key);
@@ -1667,8 +1816,10 @@ static void removeRefCountKey(SbkObject *self, const char *key)
     }
 }
 
-void keepReference(SbkObject *self, const char *key, PyObject *referredObject, bool append)
+void keepReference(SbkObject *self, const char *keyC, PyObject *referredObject, bool append)
 {
+    std::string key(keyC);
+
     if (isNone(referredObject)) {
         removeRefCountKey(self, key);
         return;
@@ -1700,7 +1851,7 @@ void keepReference(SbkObject *self, const char *key, PyObject *referredObject, b
 void removeReference(SbkObject *self, const char *key, PyObject *referredObject)
 {
     if (!isNone(referredObject))
-        removeRefCountKey(self, key);
+        removeRefCountKey(self, std::string(key));
 }
 
 void clearReferences(SbkObject *self)
@@ -1723,6 +1874,11 @@ static std::vector<PyTypeObject *> getBases(SbkObject *self)
         : std::vector<PyTypeObject *>(1, Py_TYPE(self));
 }
 
+static bool isValueType(SbkObject *self)
+{
+    return PepType_SOTP(Py_TYPE(self))->type_behaviour == BEHAVIOUR_VALUETYPE;
+}
+
 void _debugFormat(std::ostream &s, SbkObject *self)
 {
     assert(self);
@@ -1746,6 +1902,8 @@ void _debugFormat(std::ostream &s, SbkObject *self)
         s << " [validCppObject]";
     if (d->cppObjectCreated)
         s << " [wasCreatedByPython]";
+    s << (isValueType(self) ? " [value]" : " [object]");
+
     if (d->parentInfo) {
         if (auto *parent = d->parentInfo->parent)
             s << ", parent=" << reinterpret_cast<PyObject *>(parent)->ob_type->tp_name
@@ -1776,8 +1934,9 @@ std::string info(SbkObject *self)
     s << "hasOwnership...... " << bool(self->d->hasOwnership) << "\n"
          "containsCppWrapper " << self->d->containsCppWrapper << "\n"
          "validCppObject.... " << self->d->validCppObject << "\n"
-         "wasCreatedByPython " << self->d->cppObjectCreated << "\n";
-
+         "wasCreatedByPython " << self->d->cppObjectCreated << "\n"
+         "value......        " << isValueType(self) << "\n"
+         "reference count... " << reinterpret_cast<PyObject *>(self)->ob_refcnt << '\n';
 
     if (self->d->parentInfo && self->d->parentInfo->parent) {
         s << "parent............ ";
